"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("./common");
function createLink(api, store) {
    const { runtime: { sendMessage, onMessage } } = api;
    window.__store__ = {
        dispatch: store.dispatch,
        subscribe: store.subscribe,
        getState: store.getState,
        replaceReducer() {
            throw new Error('store#replaceReducer is not currently a supported operation in ext-link');
        },
    };
    store.subscribe(() => {
        const state = store.getState();
        sendMessage({
            __type__: common_1.SUBSCRIBE_TYPE,
            __target__: common_1.TARGET_FOREGROUND,
            __state__: state,
        });
    });
    onMessage.addListener((message, _, sendResponse) => {
        if (typeof message !== 'object')
            return;
        if (message === null)
            return;
        if (typeof message.__type__ !== 'string')
            return;
        if (typeof message.__target__ !== 'string')
            return;
        if (message.__target__ !== common_1.TARGET_BACKGROUND)
            return;
        if (typeof message.__action__ !== 'object')
            return;
        if (message.__action__ === null)
            return;
        switch (message.__type__) {
            case common_1.DISPATCH_TYPE: {
                const { __action__ } = message;
                store.dispatch(__action__);
                break;
            }
            case common_1.INIT_TYPE: {
                const initialState = store.getState();
                sendResponse({
                    __initialState__: initialState,
                });
                break;
            }
            default: break;
        }
    });
}
exports.default = createLink;
//# sourceMappingURL=background.js.map