"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("./common");
function onMessageListener(message) {
    // Validation
    if (typeof message !== 'object')
        return;
    if (message === null)
        return;
    if (typeof message.__type__ !== 'string')
        return;
    if (typeof message.__target__ !== 'string')
        return;
    if (message.__type__ !== common_1.SUBSCRIBE_TYPE)
        return;
    if (message.__target__ !== common_1.TARGET_FOREGROUND)
        return;
    if (typeof message.__state__ !== 'object')
        return;
    if (message.__state__ === null)
        return;
    // Reassign state
    this.prevState = this.state;
    this.state = message.__state__;
    // Notify listeners async
    this.listeners.forEach(l => setTimeout(() => l(), 0));
}
function createDispatch(context) {
    const dispatch = function (action) {
        this.sendMessage({
            __type__: common_1.DISPATCH_TYPE,
            __action__: action,
            __target__: common_1.TARGET_BACKGROUND,
        });
        return action;
    };
    return dispatch.bind(context);
}
function createSubscribe(context) {
    const subscribe = function (listener) {
        this.listeners = [...this.listeners, listener];
        return () => {
            this.listeners = this.listeners.filter(l => l !== listener);
        };
    };
    return subscribe.bind(context);
}
function createGetState(context) {
    const getState = function () {
        return this.state;
    };
    return getState.bind(context);
}
function createReplaceReducer() {
    return (_) => {
        throw new Error('store#replaceReducer is not currently a supported operation in ext-link');
    };
}
function createForegroundLink(api) {
    return __awaiter(this, void 0, void 0, function* () {
        let store;
        const { runtime: { sendMessage, onMessage, getBackgroundPage } } = api;
        const background = yield new Promise((resolve) => {
            getBackgroundPage(bg => resolve(bg));
        });
        if (background) {
            // For performance reasons bypass messaging if possible
            store = background.__store__;
        }
        else {
            const initialState = yield new Promise((resolve) => {
                sendMessage({
                    __type__: common_1.INIT_TYPE,
                    __target__: common_1.TARGET_BACKGROUND,
                }, ({ __initialState__ }) => {
                    resolve(__initialState__);
                });
            });
            const context = {
                sendMessage,
                state: initialState,
                prevState: {},
                listeners: [],
            };
            store = {
                dispatch: createDispatch(context),
                subscribe: createSubscribe(context),
                getState: createGetState(context),
                replaceReducer: createReplaceReducer(),
            };
            onMessage.addListener(onMessageListener.bind(context));
        }
        return store;
    });
}
exports.default = createForegroundLink;
//# sourceMappingURL=foreground.js.map